<?php
namespace FreePBX\modules;

class Pt1csmartivr extends \FreePBX_Helpers implements \BMO
{

    private $freepbx;

    public function __construct($freepbx = null)
    {
        parent::__construct($freepbx);
        if ($freepbx === null) {
            throw new Exception('Not given a FreePBX Object');
        }
        $this->freepbx = $freepbx;
        $this->db      = $freepbx->Database;
    }

    public function install()
    {
    }

    public function uninstall()
    {
    }

    public function backup()
    {
    }

    public function restore($backup)
    {
    }

    public function doConfigPageInit($page)
    {
    }

    /**
     * Ajax Request for BMO
     *
     * @param string $command [description]
     * @param [type] $setting [description]
     *
     * @return bool
     */
    public function ajaxRequest($command, &$setting)
    {
        switch ($command) {
            case 'check':
            case 'edit':
                return true;
            default:
                return false;
        }
    }

    /**
     * Ajax Module for BMO
     */
    public function ajaxHandler()
    {
        switch ($_REQUEST['command']) {
            case 'check':
                return $this->testModule();
                break;
            case 'edit':
                $this->saveConfig($_POST);

                return [
                    'status' => true,
                ];
            default:
                return false;
        }
    }

    /**
     * Тестирование работы модуля
     * Запрос к вебсервису 1С
     * Добавляем уникальный номер в составе запроса
     * Генерируем речь с помощью TTS
     * Возвращаем статус, если все прошло ОК
     */
    public function testModule()
    {
        $ivr = new Pt1csmartivr\lib\AGICallLogic();

        return $ivr->selfTest();
    }

    /**
     * Сохранение данных из формы в базу данных
     *
     * @param $post
     *
     * @return |null
     */
    public function saveConfig($post)
    {
        $sql = 'UPDATE pt1csmartivr SET `server1chost` = ?, `server1cport` = ?, `login` = ?, `secret` = ?, `timeout_extension` = ?, `failover_extension` = ?, `database` = ?, `useSSL` = ?, `number_of_repeat` = ?, `library_1c` = ?, `internal_ext_length` = ?, `debug_mode` = ?';

        $prepStatement = $this->db->prepare($sql);
        $values        = [
            $post['server1chost'],
            $post['server1cport'],
            $post['login'],
            $post['secret'],
            $post['timeout_extension'],
            $post['failover_extension'],
            $post['database'],
            $post['useSSL'],
            $post['number_of_repeat'],
            $post['library_1c'],
            $post['internal_ext_length'],
            $post['debug_mode'],

        ];
        $prepStatement->execute($values);

        return $this->getConfig();
    }

    /**
     * Получение данных из базы в форму
     *
     * @return |null
     */
    public function getConfig($var = null, $id = 'noid')
    {
        $sql = 'SELECT * FROM pt1csmartivr';
        $sth = $this->db->prepare($sql);
        $sth->execute();
        $data = $sth->fetch(\PDO::FETCH_ASSOC);
        $this->create_manager($data);

        return $data;
    }

    /**
     *
     * Creates the manager connection if it does not exist
     *
     */
    private function create_manager($data)
    {
        //Check if a manager profile exists for smartivr if not create it.
        $managerFound = false;
        if ((function_exists('manager_list')) && (($managers = manager_list()) !== null)) {

            //Search for smartivrusr manager
            foreach ($managers as $manager) {
                if ($manager['name'] === $data['amiUser']) {
                    $managerFound = true;
                    break;
                }
            }
        }
        //If not found create a manager profile for smartivr
        if ((function_exists("manager_add")) && ( ! $managerFound)) {
            out(_("Create manager API 'smartivrusr'"));
            manager_add($data['amiUser'], $data['amiPassword'], '0.0.0.0/0.0.0.0', '127.0.0.1/255.255.255.255',
                '', 'call,command');

            if (function_exists('manager_gen_conf')) {
                manager_gen_conf();
            }
        }
    }

    /**
     * Отображение страницы с настройками модуля
     *
     * @return string
     */
    public function showPage()
    {
        $data_value = $this->getConfig();

        return load_view(__DIR__ . '/views/settings.php', ['data_value' => $data_value]);
    }
}
