var theForm = document.edit;

if (typeof theForm !== 'undefined') {
	theForm.server1chost.focus();
}

const ModuleSmartIVR = {
	initialize() {
		// Подписка на изменнеие режима веб сервиса
		$('select[name="library_1c"]').change(function () {
			ModuleSmartIVR.onLibrary1CChange();
		});
		ModuleSmartIVR.onLibrary1CChange();

		// Отправка формы для сохранения настроек
		$('#submit-button').on('click', function (e) {
			e.preventDefault();
			ModuleSmartIVR.saveSettings();
		});

		// Проверка работы модуля
		$('#check-button').on('click', function (e) {
			e.preventDefault();
			ModuleSmartIVR.selfTest();
		});

	},
	/**
	 * Переключение типа библиотеки 1С
	 */
	onLibrary1CChange(){
		//Скроем параметры настройки сервера, если выбран режим панели 2.0
		if (theForm.library_1c.value === "1.0") {
			$('tr.only-library-pt1c-ver1').show();
			$('tr.only-library-pt1c-ver2').hide();
		} else {
			$('tr.only-library-pt1c-ver2').show();
			$('tr.only-library-pt1c-ver1').hide();
		}
	},
	/**
	 * Тестирование модуля, с выводом сообщения о результатах тестирования
	 * под кнопками и в лог файл
	 */
	selfTest() {
		$('#self-test').html('');
		$('#check-button').find('i').addClass('fa-spin');

		// Тест работы сервиса
		$.post(
			FreePBX.ajaxurl,
			{
				module: "pt1csmartivr",
				command: "check",
			}
		).done(function (data) {
			$('#check-button').find('i').removeClass('fa-spin');
			if (data.result) {
				fpbxToast('Успешное соединение с 1С и Text-To-Speech сервисом', 'Модуль работает корректно', 'success')
			} else {
				var visualErrorString = '';
				if (typeof (data.msg_text) === 'string') {
					visualErrorString = data.msg_text;
				} else if (Array.isArray(data.msg_text)) {
					$.each(data.msg_text, (index, value) => {
						visualErrorString += `${value} <br>`;
					});
				} else {
					visualErrorString = JSON.stringify(data, null, '\t');
				}
				visualErrorString = visualErrorString.replace(/\n/g, '<br/>');
				var html = `<div class="alert alert-danger">
					<div class="text-center">Ошибки при проверке работы модуля</div>
						<p>${visualErrorString}</p>
						<p>Подробности можно найти в логах в файле <a href="/admin/config.php?display=logfiles" target="_blank">ModuleSmartIvr.log</a></p>
					</div>`;
				$('#self-test').html(html);
			}

		});
	},
	/**
	 * Валидация формы перез сохранением настроек
	 * @returns {boolean}
	 */
	validateForm() {
		if (theForm.library_1c.value === "1.0") {
			if (isEmpty(theForm.server1chost.value))
				return warnInvalid(theForm.server1chost, "Не заполнен адрес сервера 1С");

			if (!isInteger(theForm.server1cport.value))
				return warnInvalid(theForm.server1cport, "Не заполнен порт для связи с 1С");

			if (isEmpty(theForm.database.value))
				return warnInvalid(theForm.database, "Не заполнен параметр точки публикации вебсервиса 1С");

			if (isEmpty(theForm.login.value))
				return warnInvalid(theForm.login, "Не заполнен логин от вебвервиса 1С");

			if (isEmpty(theForm.secret.value))
				return warnInvalid(theForm.secret, "Не заполнен пароль от вебвервиса 1С");
		}

		setDestinations(edit, 1);
		defaultEmptyOk = false;
		if (!validateDestinations(edit, 1, true)) {
			return false;
		}

		setDestinations(edit, 2);
		if (!validateDestinations(edit, 2, true)) {
			return false;
		}

		if (!isInteger(theForm.internal_ext_length.value))
			return warnInvalid(theForm.internal_ext_length, "Не заполнена длина внутреннего номера");

		if (!isInteger(theForm.number_of_repeat.value))
			return warnInvalid(theForm.number_of_repeat, "Не заполнено количество повторов меню");
		return true;
	},
	/**
	 * Сохранение настроек из формы
	 * @returns {boolean}
	 */
	saveSettings() {
		if (ModuleSmartIVR.validateForm() === false) {
			return false;
		}
		$('input[name="timeout_extension"]').val(theForm[`${theForm.goto0.value}0`].value);
		$('input[name="failover_extension"]').val(theForm[`${theForm.goto1.value}1`].value);

		$('#submit-button').blur();
		$('#submit-button').prop("disabled", true);
		$('#submit-button').text("Сохраняем...");
		var frmdata = $('form').serialize();
		$.post('ajax.php?module=pt1csmartivr&command=edit', frmdata)
			.done(function (data) {
				if (data.status) {
					fpbxToast('Настройки сохранены', '', 'success');
					toggle_reload_button("show");
				}
			})
			.fail(function (jqXHR, status, error) {
				if (typeof jqXHR.responseJSON !== undefined) {
					fpbxToast('Error: ' + jqXHR.responseJSON.error.message);
				} else {
					fpbxToast('Unknown Error');
				}
			})
			.always(function () {
				$('#submit-button').prop("disabled", false);
			});
		return true;
	}
};

$(document).ready(function () {
	ModuleSmartIVR.initialize();
});