<?php
/**
 * Copyright (C) MIKO LLC - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Nikolay Beketov, 2 2020
 *
 */


/**
 * Copyright © MIKO LLC - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Alexey Portnov, 9 2019
 */
namespace FreePBX\modules\Pt1csmartivr\lib;

use Exception;
use FreePBX;
use AGI_AsteriskManager;
use AGI;

/**
 * Class SmartIVR
 */
class AGICallLogic
{
    private $agi = null;
    private $module_name = 'ModuleSmartIVR';
    public  $ttsDir = '';
    private $internalExtLength = 3; // Длинна внутреннего номера.
    private $contextInternal = 'from-internal';
    private $number;
    private $messages = [];
    private $web_service_1C;
    private $count_of_repeat_ivr;
    private $timeout_extension;
    private $failover_extension;
    private $module_extension;
    private $logger;
    private $amiUser = 'smartivrusr';
    private $amiPassword = 'dsjfhsdjfhj34283dhs';

    /**
     * SmartIVR constructor.
     *
     * @throws \Exception
     */
    public function __construct()
    {
        $this->logger     = new Logger('SmartIVR', $this->module_name);
        $this->number     = '74952293042'; //  Для тестирования, норм потом в AGI он переопределиться

        // Каталог генерированных файлов.
        $this->ttsDir = FreePBX::Config()->get('ASTSPOOLDIR').'/monitor/text2speech/';
        if ( ! is_dir($this->ttsDir) && ! mkdir($concurrentDirectory = $this->ttsDir) && ! is_dir($concurrentDirectory)) {
           $this->logger->writeError(sprintf('Directory "%s" was not created', $concurrentDirectory));
            throw new Exception(sprintf('Directory "%s" was not created', $concurrentDirectory));
        }
        $pt1c = FreePBX::Pt1csmartivr();
        $settingsArr = $pt1c->getConfig();
        // Получим настройки подключения к 1С.
        if(count($settingsArr)===0){
            $this->logger->writeError('Settings not found');
            throw new Exception('FreePBX database connection error');
        } else {
            $this->module_extension = $settingsArr['extension'];
            $this->timeout_extension  = $settingsArr['timeout_extension'];
            $this->failover_extension = $settingsArr['failover_extension'];
            $this->internalExtLength  = $settingsArr['internal_ext_length'];
            // Количество повторов меню перед переводом на резервный номер
            $this->count_of_repeat_ivr = 3;
            if (isset($settingsArr['number_of_repeat'])
                && $settingsArr['number_of_repeat'] > 0) {
                $this->count_of_repeat_ivr = $settingsArr['number_of_repeat'];
            }
            $this->amiUser = $settingsArr['amiUser'];
            $this->amiPassword = $settingsArr['amiPassword'];

            $params1C = [
              'database'=>$settingsArr['database'], // Имя базы 1С, как она опубликована на web сервере.
              'login'   =>$settingsArr['login'],  // Имя пользователя для авторизации на вебсервисе 1С.
              'secret'   =>$settingsArr['secret'],    // Пароль для авторизации на вебсервисе 1С.
              'server_1c_host'   =>$settingsArr['server1chost'], // Адрес сервера, где опубликован web сервис 1С.
              'server_1c_port'   =>$settingsArr['server1cport'],  // Порт сервера, где опубликован web сервис 1С.
              'library_1c'   =>$settingsArr['library_1c'], // Версия библиотеки 1С
              'useSSL'   =>$settingsArr['useSSL']==='1', // Использовать HTTPS соединение с 1С
              'logger' => $this->logger
            ];
            $this->logger->debug = $settingsArr['debug_mode']==='1';
            $this->web_service_1C = new WebService1C($params1C);
        }
    }

    /**
     * Начало работы IVR.
     */
    public function startIVR()
    {
        $this->agi    = new AGI();
        $did = $this->agi->get_variable('FROM_DID', true);
        $this->number = $this->agi->request['agi_callerid'];
        if(empty($did) && mb_strlen($this->number) <= 4){
            $this->Verbose('Use 777777777 as caller id because the call from short number');
            // Это тестовый внутренний вызов;
            $this->number = '777777777';
        }
        $this->Verbose("Preparing IVR for {$this->number}");
        $DialStatus = '';
        $this->agi->exec('Ringing', '');
        $this->agi->set_variable('AGIEXITONHANGUP', 'yes');
        $this->agi->set_variable('AGISIGHUP', 'yes');
        $this->agi->set_variable('__ENDCALLONANSWER', 'yes');

        // Добавим вызываемый номер для модуля статистики.
        $v = $this->agi->get_variable('miko_accountcode');
        $accountCode = '';
        if(!$v['result'] === 0) {
            $accountCode = $v['data'];
        }
        if($accountCode === ''){
            $accountCode = $this->agi->request['agi_dnid'];
        }
        $this->agi->set_variable('CDR(accountcode)', $accountCode);

        // Если мы тестировали сервис, то CDR запись создавать не надо.
        if ($this->number==='777777777') {
            $this->agi->exec('NoCDR', '');
        }

        $ivr_menu_text = $this->web_service_1C->getIvrMenuText($this->number);
        $ivr_menu_file = null;
        if ($ivr_menu_text !== null) {
            $ivr_menu_file = $this->text2speech($ivr_menu_text);
        }
        $this->agi->Answer();
        if ($ivr_menu_file === null) {
            $this->Verbose("Error - redirect to failover extension {$this->failover_extension}");
            $this->endWork($DialStatus, $this->failover_extension);
            return;
        }

        for ($i = 0; $i < $this->count_of_repeat_ivr; $i++) {
            // Канал может измениться, снова установим переменную.
            $this->agi->set_variable('CDR(accountcode)', $accountCode);
            // Заглушка, иначе получаем не корректный ответ на get_data.
            $resultNull  = $this->agi->get_data('silence/1', 1, $this->internalExtLength);
            // Запрашиваем ввод внутреннего номера.
            $this->Verbose('Said -> IVR menu text message from CRM');
            $result      = $this->agi->get_data($ivr_menu_file, 3000, $this->internalExtLength);
            $selectednum = $result['result'];
            if (empty($selectednum)) {
                continue;
            }
            $this->Verbose("User enter extension - $selectednum -");
            $status = $this->getExtensionStatus($selectednum);
            if ($status === -1) {
                $this->Verbose('Said -> extension UNKNOWN');
                $text     = $this->getIVRPhrase('UnknownNumber');
                $filename = $this->text2speech([urlencode($text)]);
                $this->agi->stream_file('silence/1');
                $this->agi->stream_file($filename);
                $this->count_of_repeat_ivr++;
            } elseif ($status === 2 || $status === 1) {
                $this->Verbose('Said -> extension busy');
                $text = $this->getIVRPhrase('ExtensionBusy');
                $file = $this->text2speech([urlencode($text)]);
                $this->agi->stream_file('silence/1');
                $this->agi->stream_file($file);
                $this->count_of_repeat_ivr++;
            } elseif ($status===4) {
                $this->Verbose('Said -> extension unavailable');
                $text = $this->getIVRPhrase('ExtensionUnavailable');
                $file = $this->text2speech([urlencode($text)]);
                $this->agi->stream_file('silence/1');
                $this->agi->stream_file($file);
                $this->count_of_repeat_ivr++;
            } else {
                $this->Verbose('Said -> connect to extension');
                $text     = $this->getIVRPhrase('ConnectionToExtension', $selectednum);
                $filename = $this->text2speech([urlencode($text)]);
                $this->agi->stream_file('silence/1');
                $this->agi->stream_file($filename);

                $this->agi->set_variable('__pt1c_UNIQUEID', '');
                //$this->agi->exec('Dial', "Local/{$selectednum}@{$this->contextInternal}/n,30," . 'TtekKHhU(dial_answer)b(dial_create_chan,s,1)');
                $this->agi->exec('Dial', "Local/$selectednum@$this->contextInternal/n,30,Tt");

                $DialStatus = $this->getDialStatus();
                $this->Verbose("Dial status after connect ({$DialStatus})");
                if ('ANSWER' === strtoupper($DialStatus)) {
                    $this->Verbose('Call answered the script sends HANGUP command to PBX');
                    $this->agi->hangup();
                    return;
                } else {
                    $this->Verbose('Call did not answer continue the IVR logic');
                }
                $text     = $this->getIVRPhrase('TryAgain');
                $filename = $this->text2speech([urlencode($text)]);
                $this->agi->stream_file('silence/1');
                $this->agi->stream_file($filename);
                $this->count_of_repeat_ivr++;
            }
        }
        $this->endWork($DialStatus, $this->timeout_extension);
    }

    /**
     * Обработка завершения работы скрипта.
     *
     * @param string $DialStatus
     * @param string $extension - номер куда отправим звонок после завершения работы скрипта
     */
    private function endWork($DialStatus, $extension)
    {
        $this->Verbose('Return the IVR logic');
        // Переадресация на резервный номер.
        $DialStatus = strtoupper($DialStatus);
        if ('ANSWER' !== $DialStatus ) {
            $this->Verbose("Redirect call to the default route ({$extension}).");
            $this->agi->exec('goto', $extension);
        } else {
            $this->Verbose('Call answered the script sends HANGUP command to PBX');
            $this->agi->hangup();
        }
    }


    /**
     * Проверяет, существует ли добавочный номер в системе и возвращает его статус.
     *
     * @param        $number
     *
     * @return int
     */
    private function getExtensionStatus($number)
    {
        /**
         * -1 = Extension not found
         * 0 = Idle
         * 1 = In Use
         * 2 = Busy
         * 4 = Unavailable
         * 8 = Ringing
         * 16 = On Hold
         */
        $ami = new AGI_AsteriskManager();
        $rescon = $ami->connect('127.0.0.1', $this->amiUser, $this->amiPassword);
        if ( $rescon === false) {
            $this->Verbose("Ошибка подключения к AMI. User: $this->amiUser.");
            return -1;
        }

        $res    = $ami->ExtensionState($number, $this->contextInternal);
        $json_status=json_encode($res);
        $this->Verbose("AMI GET ExtensionState: $json_status");

        $status = (int)$res['Status'];
        $this->Verbose("Extension {$number} state is -> $status. JSON:$json_status");
        $ami->disconnect();

        return $status;
    }

    /**
     * Проверяем статус соединения с абонентом
     */
    private function getDialStatus(){
        $res = $this->agi->get_variable('DIALSTATUS');
        $json_status=json_encode($res);
        $this->Verbose("AGI GET VARIABLE ANSWER: $json_status");
        if ($res['result']==='1'){
            $DialStatus = $res['data'];
        } else {
            $DialStatus = 'UNKNOWN';
        }
        return $DialStatus;
    }


    /**
     * Генерация речи универсальная, в зависимости от выбранного сервиса TTS запускает нужную функцию.
     *
     * @param      $arr_text_to_speech
     *
     * @return null|string
     */
    private function text2speech($arr_text_to_speech)
    {
        $ttsSettings = $this->web_service_1C->getTTSSettings();
        if ($ttsSettings === null) {
            $this->Verbose('Error on get TTS settings from CRM');

            return null;
        }
        $settings = [
            'ttsDir'=>$this->ttsDir,
            'auth' =>$ttsSettings->auth,
            'logger'=>$this->logger
        ];

        switch ($ttsSettings->tts_service) {
            case 'Yandex':
            {
                $tts = new YandexTTS($settings);
                break;
            }
            case 'CRT':
            {
                $tts = new SpeechProTTS($settings);
                break;
            }
            default:
                $this->Verbose("Unknown TTS service: $ttsSettings->tts_service");

                return null;
        }
        $result = $tts->Synthesize($arr_text_to_speech, $ttsSettings->dictor);
        if (empty($result)) {
            $messages = $tts->getMessages();
            foreach ($messages as $message) {
                $this->Verbose($message);
            }
            $result = null;
        }

        return $result;
    }

    /**
     * Генерирует языкозависимую версию фразы для IVR меню по идентификатору
     *
     * @param      $phraseId  - id фразы
     * @param null $extension - внутренний номер
     *
     * @return string
     */
    private function getIVRPhrase($phraseId, $extension = null)
    {
        $resultText = '';
        switch ($phraseId) {
            case 'UnknownNumber':
            {
                $resultText = 'Не правильно набран номер';
                break;
            }
            case 'ExtensionBusy':
            {
                $resultText = 'Абонент сейчас разговаривает, попробуйте позвонить позднее или введите другой номер';
                break;
            }
            case 'ExtensionUnavailable':
            {
                $resultText = 'Абонент недоступен, попробуйте позвонить позднее или введите другой номер';
                break;
            }
            case 'ConnectionToExtension':
            {
               $resultText = 'Соединяю с номером ' . $extension;
               break;
            }
            case 'TryAgain':
            {
                $resultText = 'Не вышло связаться с абонентом. Попробуйте еще раз.';
                break;
            }
            default:
                break;
        }

        return $resultText;
    }

    /**
     * Вывести сообщение об отладке
     *
     * @param $value
     */
    public function Verbose($value)
    {
        $this->messages[] = $value;
        if ($this->agi !== null) {
            $this->agi->verbose('SMART IVR VERBOSE: '.escapeshellarg($value), 3);
        }
        $this->logger->writeInfo($value);
    }

    /**
     * Функция самотестирования
     * При вызове отправляет запрос к 1С на генерацию текста
     * В текст добавляется уникальное число и он отправляется на генерацию
     *
     * Вызывается при каждом редактировании настроек модуля
     * @return array
     * @throws \Exception
     */
    public function selfTest()
    {
        $this->logger->debug = true;
        $filename    = null;
        $ivrMenuText = $this->web_service_1C->getIvrMenuText($this->number);
        if (is_array($ivrMenuText)) {
            // При тестировании надо всегда работать с уникальной строкой, чтобы не попадать в кеш
            $ivrMenuText[] = random_int(1, 9999);
            $filename      = $this->text2speech($ivrMenuText);
        } else {
            $messagesFrom1C = $this->web_service_1C->getMessages();
            foreach ($messagesFrom1C as $message) {
                $this->Verbose($message);
            }
        }

        $result_data = [
            'IVR'      => $filename,
            'msg_text' => $this->messages,
            'result'   => $filename !== null
        ];

        // Отправим в 1С информацию о внутреннем номере модуля.
        // В FreePBX он фиксированный, в MikoPBX произвольный
        $this->web_service_1C->sendModuleExtension($this->module_extension);

        return $result_data;

    }
}